#!/usr/bin/python
#-------------------------------python-----------------------esbase.py--#
#                                                                       #
#                    Classe base pe gli esercizi                        #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2014---#


"""
Base class for exercise.
"""

from __future__ import print_function, division
import random
#import math
from fract import Fraction, mcd #, FRA_TYPE
from polynx import Polynx
from symbbase import MultiOutputBase
from abc import abstractmethod


FRA_TYPE = type(Fraction())
ODT, TEX, PY, TXT = range(4)
SEPARATED, INLINE, MOODLEXML = range(3)

XMLDOC = r"""<?xml version="1.0" ?>
<quiz>
{0}
</quiz>
"""

XMLITEM = r"""    <question type="multichoice">
        <name>
            <text>{0}</text>
        </name>
        <questiontext format="html">
            <text>{1}</text>
        </questiontext>
        <answer fraction="100">
            <text>{2}</text>
            <feedback><text>Correct!</text></feedback>
        </answer>
        <answer fraction="100">
            <text>{2}</text>
            <feedback><text>Correct!</text></feedback>
        </answer>
        <answer fraction="100">
            <text>{2}</text>
            <feedback><text>Correct!</text></feedback>
        </answer>
        <answer fraction="100">
            <text>{2}</text>
            <feedback><text>Correct!</text></feedback>
        </answer>
        <answer fraction="100">
            <text>{2}</text>
            <feedback><text>Correct!</text></feedback>
        </answer>
        <shuffleanswers>1</shuffleanswers>
        <single>true</single>
        <answernumbering>abc</answernumbering>
    </question>"""

####
# Tools
###

def near(num0, num1):
    """num0 near num1."""
    return abs(num0 - num1) < 1e-8

def rrnoval(val_mi, val_ma, excluded=[]):
    """Return a random value not in v."""
    while True:
        result = random.randrange(val_mi, val_ma)
        if not result in excluded:
            return result

def randsign():
    """Return a random sign."""
#    return rrnoval(-1, 2, [0])
    if random.randrange(0, 2):
        return -1
    else:
        return +1
    
def coppiaprimi(val_mi, val_ma, noa=[], nob=[]):
    """Pair of random prime integer."""
    num0 = rrnoval(val_mi, val_ma, noa)
    num1 = rrnoval(val_mi, val_ma, nob)
    div = mcd(num0, num1)
#  print a, b, d
    return num0 // div, num1 // div

def binomio(grado, coe1, coe0):
    """Binomial."""
    return Polynx([coe1] + [0] * (grado - 1) + [coe0])

def odd(num):
    """num is odd."""
    return num % 2

def even(num):
    """num is even."""
    return num % 2 - 1

def divisors(num):
    """Result a tuple with divisors of num."""
    if type(num) == FRA_TYPE:
        num = num.getnum()
    num = abs(num)
    result = []
    div = 1
    while True:
        quo, rem = divmod(num, div)
    #    print d, q, r
        if div > quo:
            break
        if div == quo:
            result.append(div)
        elif rem == 0:
            result.extend([div, quo])
        div += 1
    return result

#def delta(a, b, c):
#    """Discr."""
#    return b * b - 4 * a * c

##def rd(a, b, c):
##  return sqrt(-delta(a, b, c))

def eq2solve(c_a, c_b, c_c):
    """Return solutions of eq2."""
#  print a, b, c
#    d = delta(a, b, c)
    discr = c_b * c_b - 4 * c_a * c_c
#  print d
    if discr >= 0:
        sqrt_discr = discr ** .5
#    print rd
#    print (-b-rd)/(2*a), (-b+rd)/(2*a)
        return ((-c_b - sqrt_discr) / (2 * c_a),
                (-c_b + sqrt_discr) / (2 * c_a))
    else:
        sqrt_discr = (-discr) ** .5
        p_real = Fraction(-c_b, 2 * c_a)
        p_imm = Fraction(sqrt_discr, 2 * c_a)
        if p_imm >= 0:
            return ("{0} + {1}j".format(p_real, p_imm),
                    "{0} - {1}j".format(p_real, p_imm))
        else:
            return ("{0} - {1}j".format(p_real, -p_imm),
                    "{0} + {1}j".format(p_real, -p_imm))
    
def det2(a00, a01, a10, a11):
    """Return det to matrix3.
a00, a01,
a10, a11,
    """
    return (a00 * a11 - a01 * a10)

def solsist2(eq1, eq2):
    """Return solutions of linear sist."""
    cx1, cy1, tn1 = eq1
    cx2, cy2, tn2 = eq2
    det0 = det2(cx1, cy1, cx2, cy2)
    if det0:
        return (Fraction(det2(tn1, cy1, tn2, cy2), det0),
                Fraction(det2(cx1, tn1, cx2, tn2), det0))
    else:
        return (None, None)

def det3(a00, a01, a02, a10, a11, a12, a20, a21, a22):
    """Return det to matrix3.
a00, a01, a02,
a10, a11, a12,
a20, a21, a22
    """
    return (a00 * a11 * a22 + a01 * a12 * a20 + a02 * a10 * a21 - 
            a00 * a12 * a21 - a01 * a10 * a22 - a02 * a11 * a20)

def solsist3(eq1, eq2, eq3):
    """Return solutions of linear sist."""
    cx1, cy1, cz1, tn1 = eq1
    cx2, cy2, cz2, tn2 = eq2
    cx3, cy3, cz3, tn3 = eq3
    det0 = det3(cx1, cy1, cz1, cx2, cy2, cz2, cx3, cy3, cz3)
    if det0:
        return (Fraction(det3(tn1, cy1, cz1, tn2, cy2, cz2, tn3, cy3, cz3),
                         det0),
                Fraction(det3(cx1, tn1, cz1, cx2, tn2, cz2, cx3, tn3, cz3),
                        det0),
                Fraction(det3(cx1, cy1, tn1, cx2, cy2, tn2, cx3, cy3, tn3),
                        det0))
    else:
        return (None, None, None)

def solsist(*equations):
    """Return solutions of linear sist."""
    if len(equations) == 2:
        return solsist2(*equations)
    elif len(equations) == 3:
        return solsist3(*equations)
    else:
        return None

####
# Base class ofr Execises
###

class EsbaseError(Exception):
    """Fraction error."""


class Exebase(MultiOutputBase):
    """Base class for exercises."""
    
    def __init__(self):
        self.delivery = "l'espressione seguente equivale a:"
        self.exe = None
        self.res = None

    def __str__(self):
        return '{0}\t{1}'.format(self.exe2str(), self.res2str())

    @abstractmethod
    def exe2str(self):
        """Abstract method for Exebase."""

    @abstractmethod
    def res2str(self):
        """Abstract method for Exebase."""

class Workbook(MultiOutputBase):
    """Workbook."""

    layout = 1            # INLINE | SEPARATED

    def __init__(self):
        self.num = 0
        self.items = []

    def __str__(self):
        """str."""
        if self.items != []:
            err_mess = 'format {0} not supported'
            if self.layout == INLINE:
                try:
                    return [self._str_odt_inl,
                            self._str_tex_inl][self.out_format]()
                except IndexError:
                    raise EsbaseError(err_mess.format(self.out_format))
            if self.layout == SEPARATED: #, MOODLEXML = range(3)
                try:
                    return [self._str_odt_sep,
                            self._str_tex_sep][self.out_format]()
                except IndexError:
                    raise EsbaseError(err_mess.format(self.out_format))
            if self.layout == MOODLEXML:
                return self._str_xml_moodle()
        else:
            err_mess = 'layout {0} not supported'
            raise EsbaseError(err_mess.format(self.layout))
            
    def _str_odt_inl(self):
        """.odt string inline."""
        lines = []
        for item in self.items:
            exe = item.exe2str()
            res = item.res2str()
            lines.append('"{0:3}. " {1} \t[{2}]'.format(self.num, exe, res))
            self.num += 1
        return " newline\n".join(lines)

    def _str_odt_sep(self):
        """.odt string separated."""
        lines = []
        num = self.num
        for item in self.items:
            exe = item.exe2str()
            lines.append('"{0:3}. " {1}'.format(num, exe))
            num += 1
        lines.append('newline "soluzioni:"')
        for item in self.items:
            res = item.res2str()
            lines.append('"{0:3}. " {1}'.format(self.num, res))
            self.num += 1
        return " newline\n".join(lines)

    def _str_tex_inl(self):
        """.odt string inline."""
        lines = []
        for item in self.items:
            exe = item.exe2str()
            res = item.res2str()
            lines.append(r'{0:3}. {1} = {2}'.format(self.num, exe[:-2],
                                                              res[2:]))
            self.num += 1
        return "\n\n".join(lines)

    def _str_tex_sep(self):
        """.tex string separated."""
        lines = []
        num = self.num
        for item in self.items:
            exe = item.exe2str()
            lines.append(r'{0:3}. {1}'.format(num, exe))
            num += 1
        lines.append('\n\nsoluzioni:')
        for item in self.items:
            res = item.res2str()
            lines.append(r'{0:3}. {1}'.format(self.num, res))
            self.num += 1
        return "\n\n".join(lines)

    def _str_xml_moodle(self):
        """.xml for moodle."""
        self.out_format = TEX
        return XMLDOC.format(self._str_xml_moodle_item())

    def _str_xml_moodle_item(self):
        """.xml for moodle."""
        lines = []
        for item in self.items:
            exe = item.exe2str()
            res = item.res2str()
            lines.append(XMLITEM.format(item.delivery, exe, res))
            self.num += 1
        return "\n".join(lines)

    def generates(self, num, item_class, val_mi, val_ma):
        """Return n exercises follwed by n solutions."""
        self.items = [item_class(val_mi, val_ma) for _ in range(num)]


def main():
    """Main program."""
    direse = '/exercises'
    import sys
    sys.path.append(sys.path[0] + direse)
    mod = __import__('01prodnot')
    exe = getattr(mod, 'Exe01Somdif')
    workbook = Workbook()
    workbook.generates(5, exe, -10, +10)
    for layout in range(2):
        print('\nlayout:', layout)
        workbook.layout = layout
        for out_format in range(2):
            print('\nout_format:', out_format)
            workbook.out_format = out_format
            print(workbook)
    layout = 2
    print('\nlayout:', layout)
    workbook.layout = layout
    workbook.out_format = 0
    print(workbook)
    
if __name__ == "__main__" :
    main ()
