#!/usr/bin/python
#-------------------------------python----------------------prodnot.py--#
#                                                                       #
#                         Prodotti notevoli                             #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2013---#

"""
Prodotti notevoli
"""

from __future__ import print_function, division

if __name__ == "__main__" :
    import sys
    sys.path.append(sys.path[0][:-9])

import esbase as eb
import random
import polynx as px

class _Prodbin(eb.Exebase):
    """Prodotto di polinomi."""
    def __init__(self):
        eb.Exebase.__init__(self)
        self.pol0 = None
        self.pol1 = None
        self.e_strings = [r'left ( {0} right ) left ({1} right )',
                          r'$$\left({0} \right) \left({1} \right)$$']
        self.r_strings = [r'{0}', r'$${0}$$']

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.pol0, self.pol1)

    def res2str(self):
        """Response to string."""
        return self.r_strings[self.out_format].format(self.pol0 * self.pol1)

class _Powbin(eb.Exebase):
    """Potenza di un binomio: (ax+b)^n."""
    def __init__(self, val_mi, val_ma, exp):
        eb.Exebase.__init__(self)
        a = eb.rrnoval(0, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        self.pol0 = px.Polynx([a, b])
        self.prod = self.pol0
        for _ in range(exp-1):
            self.prod *= self.pol0
        str_exp = '{0}'.format(exp)
        self.e_strings = [r'left ({0} right )^' + str_exp, 
                          r'$$\left({0} \right)^' + str_exp + '$$']
        self.r_strings = [r'{0}', r'$${0}$$']

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.pol0)

    def res2str(self):
        """Response to string."""
        return self.r_strings[self.out_format].format(self.prod)

class Exe01Somdif(_Prodbin):
    """Somma per differenza: (ax+b)(ax-b)."""
    def __init__(self, val_mi, val_ma):
        _Prodbin.__init__(self)
        a = eb.rrnoval(val_mi, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        c, d = a, -b
        self.pol0 = px.Polynx([a, b])
        self.pol1 = px.Polynx([c, d])

class Exe02Quadbin(_Powbin):
    """Quadrato di un binomio: (ax+b)^2."""
    def __init__(self, val_mi, val_ma):
        _Powbin.__init__(self, val_mi, val_ma, 2)


class Exe03Quadtrin(_Powbin):
    """Quadrato di un trinomio: (ax^2+bx+c)^2."""
    def __init__(self, val_mi, val_ma):
        _Powbin.__init__(self, val_mi, val_ma, 2)
        a = eb.rrnoval(0, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        c = eb.rrnoval(val_mi, val_ma, [0])
        self.pol0 = px.Polynx([a, b, c])

class Exe04Trinnot(_Prodbin):
    """Trinomio particolare: (x+a)(x+b)."""
    def __init__(self, val_mi, val_ma):
        _Prodbin.__init__(self)
        a = eb.rrnoval(0, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        self.pol0 = px.Polynx([1, a])
        self.pol1 = px.Polynx([1, b])

class Exe05Cubbin(_Powbin):
    """Cubo di un binomio: (ax+b)^3."""
    def __init__(self, val_mi, val_ma):
        _Powbin.__init__(self, val_mi, val_ma, 3)

class Exe06Powbin(_Powbin):
    """Potenza di un binomio: (ax+b)^n."""
    def __init__(self, val_mi, val_ma):
        _Powbin.__init__(self, val_mi, val_ma, random.randrange(3, 6))

def main():
    """Main program."""
    exercises = (Exe01Somdif,  Exe02Quadbin, 
                 Exe03Quadtrin, Exe04Trinnot, 
                 Exe05Cubbin, Exe06Powbin)
    for exe in exercises:
        print('\n', exe.__doc__)
        items = [exe(-10, +10) for _ in range(3)]
        for out_format in range(2):
            print('\nout_format:', out_format)
            items[0].out_format = out_format
            for item in items:
                print(item)
    
if __name__ == "__main__" :
    main ()
