#!/usr/bin/python
#-------------------------------python---------------------05sistli.py--#
#                                                                       #
#                   sistemi lineari di due incognite                    #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli----------------------------------------------2014---#

"""
Sistemi lineari
"""

from __future__ import print_function, division

if __name__ == "__main__" :
    import sys
    sys.path.insert(0, sys.path[0][:-10])

import esbase as eb
import polynx as px

EQNORMODF = (r'{0} x {1} y = {2}')
EQNORMTEX = (r'{0} x {1} y = {2}')
SISNORMODF = (r'''left lbrace binom
  {{""{0}}}
  {{""{1}}}
right none''')
SISNORMTEX = (r'''left lbrace binom
  {{""{0}}}
  {{""{1}}}
right none''')

class _sistlin(eb.Exebase):
    """Sistemi lineari."""
    def __init__(self, val_mi, val_ma):
        eb.Exebase.__init__(self)
        self.eq_strings = [EQNORMODF, EQNORMTEX]
        self.e_strings = [SISNORMODF, SISNORMTEX]
        self.r_strings = [r'left ({0}; {1} right )', r'$$({0}; {1})$$']
        a, b, c, d, e, f, x, y = self.create(val_mi, val_ma)
        self.sist = ((px.strcoeff(a, True), px.strcoeff(b), str(c)),
                     (px.strcoeff(d, True), px.strcoeff(e), str(f)))
        self.result = (x, y)

    def exe2str(self):
        """Exercises to string."""
        coe0, coe1 = self.sist
        eq0 = self.eq_strings[self.out_format].format(*coe0)
        eq1 = self.eq_strings[self.out_format].format(*coe1)
        return self.e_strings[self.out_format].format(eq0, eq1)

    def res2str(self):
        """Response to string."""
        return self.r_strings[self.out_format].format(*self.result)

class Exe01sistlinint(_sistlin):
    """Sistemi lineari a risultati interi."""
    def create(self, val_mi, val_ma):
        a = eb.rrnoval(val_mi, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        d = eb.rrnoval(val_mi, val_ma, [0])
        e = eb.rrnoval(val_mi, val_ma, [0])
        x = eb.rrnoval(val_mi, val_ma, [0])
        y = eb.rrnoval(val_mi, val_ma, [0])
        c = a * x + b * y
        f = d * x + e * y
        return a, b, c, d, e, f, x, y

class Exe02sistlinraz(_sistlin):
    """Sistemi lineari a risultati razionali."""
    def create(self, val_mi, val_ma):
        a = eb.rrnoval(val_mi, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        c = eb.rrnoval(val_mi, val_ma, [0])
        d = eb.rrnoval(val_mi, val_ma, [0])
        e = eb.rrnoval(val_mi, val_ma, [0])
        f = eb.rrnoval(val_mi, val_ma, [0])
        x, y = eb.solsist((a, b, c), (d, e, f))
        return a, b, c, d, e, f, x, y

def main():
    """Main program."""
    exercises = (Exe01sistlinint,  Exe02sistlinraz,)
    for exe in exercises:
        print('\n', exe.__doc__)
        items = [exe(-10, +10) for _ in range(3)]
        for out_format in range(2):
            print('\nout_format:', out_format)
            items[0].out_format = out_format
            for item in items:
                print(item)
    
if __name__ == "__main__" :
    main ()
