#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
#-------------------------------python--------------------------eq2.py--#
#                                                                       #
#                     Equazioni di secondo grado                        #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, val_ma  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2006---#

"""
Eq. di secondo grado
"""

from __future__ import print_function, division

if __name__ == "__main__" :
    import sys
    sys.path.append(sys.path[0][:-9])

import esbase as eb
#import random
from fract import Fraction
from polynx import Polynx

class _Eq2(eb.Exebase):
    """Equazione."""
    def __init__(self):
        eb.Exebase.__init__(self)
        self.delivery = "Risolvi l'equazione"
        self.e_strings = [r'{0} = 0', r'$${0} = 0$$']
        self.r_strings = [r'left [{0}; {1} right ]',
                          r'$$\left [{0}; {1} \right ]$$']

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.exe)

    def res2str(self):
        """Response to string."""
        res0, res1 = self.res
        return self.r_strings[self.out_format].format(res0, res1)
        
class _Eq2_compl(eb.Exebase):
    """Equazione."""
    def __init__(self):
        eb.Exebase.__init__(self)
        self.delivery = "Risolvi l'equazione"
        self.xre = None
        self.xim = None
        self.e_strings = [r'{0} = 0', r'$${0} = 0$$']
        self.r_strings = [r'left [{0}; {1} right ]',
                          r'$$\left [{0}; {1} \right ]$$']

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.exe)

    def res2str(self):
        """Response to string."""
        res0 = '{0} - {1} i'.format(self.xre, self.xim)
        res1 = '{0} + {1} i'.format(self.xre,self. xim)
        return self.r_strings[self.out_format].format(res0, res1)

#def exe_02_eq2cq(exe, res0, res1):  # TODO implementare per le equazioni con coefficienti razinali
#    co = random.choice(e.coeff)
#    if co:
#      di = random.choice(eb.divisors(co))
#      e/=di
#    Eq2.genera()

class Exe01eq2rzcz(_Eq2):
    """Eq. 2gr: rad. intere, coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eq2.__init__(self)
        a = eb.rrnoval(0, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        self.exe = Polynx([1, a]) * Polynx([1, b])
        self.res = (-a, -b)

class Exe02eq2rqcz(_Eq2):
    """Eq. 2gr: rad. razionali, coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eq2.__init__(self)
        a = eb.rrnoval(0, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        c = eb.rrnoval(0, val_ma, [0])
        d = eb.rrnoval(val_mi, val_ma, [0])
        self.exe = Polynx([a, b]) * Polynx([c, d])
        self.res = (Fraction(-b, a), Fraction(-d, c))

class Exe03eq2rzcq(_Eq2):
    """Eq. 2gr: rad. intere, coeff. razionali"""
    def __init__(self, val_mi, val_ma):
        _Eq2.__init__(self)
        a = eb.rrnoval(0, val_ma, [0])
        b = eb.rrnoval(val_mi, val_ma, [0])
        fac = eb.rrnoval(1, val_ma, [0])
        div = eb.rrnoval(1, val_ma, [0])
        pol0 = Polynx([Fraction(1 * fac, div), Fraction(a * fac, div)])
        pol1 = Polynx([1, b])
        self.exe = pol0 * pol1
        self.res = (-a, -b)

class Exe04eq2rqcq(_Eq2):
    """Eq. 2gr: rad. razionali, coeff. razionali"""
    def __init__(self, val_mi, val_ma):
        _Eq2.__init__(self)
        a = eb.rrnoval(1, val_ma)
        b = eb.rrnoval(val_mi, val_ma, [0])
        c = eb.rrnoval(1, val_ma)
        d = eb.rrnoval(val_mi, val_ma, [0])
        pol0 = Polynx([Fraction(a, b), 1])
        pol1 = Polynx([1, Fraction(d, c)])
        self.exe = pol0 * pol1
        self.res = (-a, -b)

class Exe05eq2rccz(_Eq2_compl):
    """Eq. 2gr: rad. complesse, coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eq2_compl.__init__(self)
        a = eb.rrnoval(1, val_ma)
        bre = eb.rrnoval(val_mi, val_ma)
        bim = eb.rrnoval(val_mi, val_ma, [0])
        sumx = bre * 2 * a
        prox = bre * bre + bim * bim
        self.xre = Fraction(bre, a)
        self.xim = Fraction(bim, a)
        self.exe = Polynx([a * a, -sumx, prox])
#        self.res = ('{0} - {1} i'.format(xre, xim),
#                    '{0} + {1} i'.format(xre, xim))

class Exe06eq2rccq(_Eq2_compl):
    """Eq. 2gr: rad. complesse, coeff. razionali"""
    def __init__(self, val_mi, val_ma):
        _Eq2_compl.__init__(self)
        a = eb.rrnoval(1, val_ma)
        bre = eb.rrnoval(val_mi, val_ma)
        bim = eb.rrnoval(val_mi, val_ma, [0])
        sumx = bre * 2 * a
        prox = bre * bre + bim * bim
        self.xre = Fraction(bre, a)
        self.xim = Fraction(bim, a)
        self.exe = Polynx([a * a, -sumx, prox])
#        self.res = ('{0} - {1} i'.format(xre, xim),
#                    '{0} + {1} i'.format(xre, xim))

def main():
    """Main program."""
    exercises = (Exe01eq2rzcz, Exe02eq2rqcz,
                 Exe03eq2rzcq, Exe04eq2rqcq,
                 Exe05eq2rccz, Exe06eq2rccq)
    for exe in exercises:
        print('\n', exe.__doc__)
        items = [exe(-10, +10) for _ in range(3)]
        for out_format in range(2):
            print('\nout_format:', out_format)
            items[0].out_format = out_format
            for item in items:
                print(item)

if __name__ == "__main__" :
    main ()
