#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
#-------------------------------python-------------------------eq2+.py--#
#                                                                       #
#              Equazioni di grado superiore al secondo                  #
#                                                                       #
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2006---#

"""
Eq. di grado superiore al secondo
"""

from __future__ import print_function, division

if __name__ == "__main__" :
    import sys
    sys.path.append(sys.path[0][:-9])

#from symbbase import MultiOutputBase
from fract import Fraction
from polynx import Polynx
import esbase as eb
#import random

class _Eqbase(eb.Exebase):
    """Equazione."""
    def __init__(self):
        eb.Exebase.__init__(self)
        self.who = None
        self.delivery = "Risolvi l'equazione"
        self.e_strings = [r'{0} = 0', r'$${0} = 0$$']
        self.r_strings = None

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.exe)

    def res2str(self):
        """Response to string."""
        

class _Eqbin(_Eqbase):
    """Equazione binomia."""
    def __init__(self):
        _Eqbase.__init__(self)
        self.index = None
        self.rad = None
        self.r_strings = [r'nroot{{{0}}}{{{1}}}',
                          r'-+ nroot{{{0}}}{{{1}}}',
                          r'\sqrt[{0}]{{{1}}}',
                          r'\mp \sqrt[{0}]{{{1}}}']

    def res2str(self):
        """Response to string."""
        i_s = self.out_format * 2 + self.who
        return self.r_strings[i_s].format(self.index, self.rad)

class _Eqtrin(_Eqbase):
    """Equazione trinomia."""
    def __init__(self):
        _Eqbase.__init__(self)
        self.r_strings = [r'left [{0}; {1} right ]',
                          r'left [-+ {0}: -+{1} right ]',
                          r'left [{0}; {1} \right ]',
                          r'\left [\mp {0}; \mp{1} \right ]']

    def res2str(self):
        """Response to string."""
        i_s = self.out_format * 2 + self.who
        return self.r_strings[i_s].format(self.res[0], self.res[1])

class _Eqter(_Eqbase):
    """Equazione di terzo grado."""
    def __init__(self):
        _Eqbase.__init__(self)
        self.r_strings = [r'left [-+ sqrt {0}; {1} right ]',
                          r'left [-+ sqrt {0}i; {1} right ]',
                          r'left [\mp sqrt {0}; {1} right ]',
                          r'left [\mp sqrt {0}i; {1} right ]']

    def res2str(self):
        """Response to string."""
        i_s = self.out_format * 2 + self.who
        return self.r_strings[i_s].format(self.res[0], self.res[1])

class Exe01eqBinrrcz(_Eqbin):
    """Eq. binomie"""
    def __init__(self, val_mi, val_ma):
        _Eqbin.__init__(self)
        f = eb.rrnoval(val_mi/2, val_ma/2, [0])
        a = eb.rrnoval(val_mi/2, val_ma/2, [0]) * f
        b = eb.rrnoval(val_mi/2, val_ma/2, [0]) * f
        self.index = eb.rrnoval(3, 6)
        if eb.odd(self.index):
            self.who = 0
        else:
            self.who = 1
        self.exe = eb.binomio(self.index, a, b)
        self.rad = Fraction(-b, a)

class Exe02eqBinrqcz(_Eqbin):
    """Eq. binomie rad. razionali coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eqbin.__init__(self)
        a, b = eb.coppiaprimi(val_mi/2, val_ma/2, [0], [0])
        f = eb.rrnoval(val_mi/2, val_ma/2, [0])
        n = eb.rrnoval(3, 6)
        an = f * a ** n
        bn = f * b ** n
        self.index = eb.rrnoval(3, 6)
        if eb.odd(self.index):
            self.who = 0
        else:
            self.who = 1
            sign = eb.randsign()
            an *= sign
            bn *= -sign
        self.index = n
        self.exe = eb.binomio(self.index, an, bn)
        self.rad = Fraction(-b, a)

class Exe03eqBiquadrzcz(_Eqtrin):
    """Eq. biquadratiche: rad. intere, coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eqtrin.__init__(self)
        a, b = eb.coppiaprimi(val_mi, val_ma, [0], [0])
        qa = a * abs(a)
        qb = b * abs(b)
        pol0 = Polynx([1, qa])
        pol1 = Polynx([1, qb])
        quad = pol0 * pol1
        coeff = quad.coeff
        self.exe = Polynx([coeff[0], 0, coeff[1], 0, coeff[2]])
        self.who = 1
        if a > 0:
            res1 = "{0}i".format(a)
        else:
            res1 = "{0}".format(-a)
        if b > 0:
            res2 = "{0}i".format(b)
        else:
            res2 = "{0}".format(-b)
        self.res = (res1, res2)

class Exe04eqTrirzcz(_Eqtrin):
    """Eq. trinomia: rad. intere, coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eqtrin.__init__(self)
        a, b = eb.coppiaprimi(val_mi/2, val_ma/2, [0], [0])
        ca = a * a * a
        cb = b * b * b
        pol0 = Polynx([1, ca])
        pol1 = Polynx([1, cb])
        quad = pol0 * pol1
        coeff = quad.coeff
        self.who = 0
        self.exe = Polynx([coeff[0], 0, 0, coeff[1], 0, 0, coeff[2]])
        self.res = (-a, -b)

class Exe05eq3rqcz(_Eqter):
    """Eq. 3gr scomponibile: rad. intere, coeff. interi"""
    def __init__(self, val_mi, val_ma):
        _Eqter.__init__(self)
        a, b = eb.coppiaprimi(val_mi/2, val_ma/2, [0], [0])
        c = eb.rrnoval(val_mi/2, val_ma/2, [0])
        d = eb.rrnoval(val_mi/2, val_ma/2, [0])
        pol0 = eb.binomio(2, a, b)
        pol1 = Polynx([c, d])
        self.exe = pol0 * pol1
        res0 = Fraction(-b, a)
        res1 = Fraction(-d, c)
        if res0 > 0:
            self.who = 0
            self.res = (res0, res1)
        else:
            self.who = 1
            self.res = (-res0, res1)

def main():
    """Main program."""
    exercises = (Exe01eqBinrrcz, Exe02eqBinrqcz, Exe03eqBiquadrzcz,
                 Exe04eqTrirzcz, Exe05eq3rqcz)
    for exe in exercises:
        print('\n', exe.__doc__)
        items = [exe(-10, +10) for _ in range(3)]
        for out_format in range(2):
            print('\nout_format:', out_format)
            items[0].out_format = out_format
            for item in items:
                print(item)
    

if __name__ == "__main__" :
    main ()
