#!/usr/bin/python
#-------------------------------python----------------------circonf.py--#
#                                                                       #
#                  Circonferenze nel piano cartesiano                   #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2014---#

#if __name__ == "__main__" :
#  import sys
#  sys.path.append(sys.path[0][:-9])

"""
Circonferenza nel piano cartesiano
"""

from __future__ import print_function, division
import sys
sys.path.append(sys.path[0][:-9])
import esbase as eb
from fract import Fraction
import polynx as px
from symbbase import MultiOutputBase

#from esbase import *

####
# Classi del piano cartesiano
###

class Point(MultiOutputBase):
    """Class for Points."""

    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __str__(self):
        return [self._str_odt, self._str_tex][self.out_format]()

    def _str_odt(self):
        """odt string."""
        return r'left ({0}; {1} right )'.format(self.x, self.y)

    def _str_tex(self):
        """tex string."""
        return r'\left ({0}; {1} \right )'.format(self.x, self.y)

    def __add__(self, other):
        return Point(self.x + other.x, self.y + other.y)

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y

    def __ne__(self, other):
        return self.x != other.x or self.y != other.y

####
# Funzioni che restituiscono punti a caso
###

def randpoint(val_mi, val_ma, notx=(), noty=(), notp=()):
    """Return a random Point."""
    while True:
        point = Point(eb.rrnoval(val_mi, val_ma, notx), 
                      eb.rrnoval(val_mi, val_ma, noty))
        if not point in notp:
            break
    return point

def three_points_not_in_line(val_mi, val_ma, notx=(), noty=()):
    """Return three random points not in line."""
    a = randpoint(val_mi, val_ma, notx, noty)
    b = randpoint(val_mi, val_ma, notx, noty, (a,))
    while True:
        c = randpoint(val_mi, val_ma, notx, noty, (a, b))
        x_a, y_a = a.x, a.y
        x_b, y_b = b.x, b.y
        x_c, y_c = c.x, c.y
        if x_b == x_a and x_c == x_b:
            continue
        elif (x_b == x_a and x_c != x_b or 
              x_c == x_b and x_b != x_a or
              (y_b-y_a) / (x_b-x_a) != (y_c-y_b) / (x_c-x_b)):
            break
    return a, b, c

####
# Line
###

class Linemq(object):
    """Clas for Line by m and q."""

    def __init__(self, m, q):
        self.m = m
        self.q = q
    
    def eval(self, x):
        """Return P(x)."""
        return self.m * x + self.q

    def __str__(self):
        if self.m == None:
            var, pol = 'x', px.Polynx(self.q)
        else:
            var, pol = 'y', px.Polynx((self.m, self.q))
        return r'{0} = {1}'.format(var, pol)
#        return [self._str_odt, self._str_tex][self.out_format](var, pol)

#    def _str_odt(self, var, pol):
#        return r'{0} = {1}'.format(self.x, self.y)
#
#    def _str_tex(self, var, pol):
#        return r'{0} = {1}'.format(self.x, self.y)


class Line(Linemq):
    """Clas for Line by two Points."""

    def __init__(self, point0, point1):
        x_0, y_0 = point0.x, point0.y
        x_1, y_1 = point1.x, point1.y
        if x_0 == x_1:
            self.m = None
            self.q = x_1
        elif y_0 == y_1:
            self.m = 0
            self.q = y_0
        else:
            self.m = Fraction(y_1 - y_0, x_1 - x_0)
            self.q = y_1-self.m * x_1

####
# Circle
###

class Circle(object):
    """Class for Circle by center and radius."""
  
    def __init__(self, center, radius):
        """Calcola i 3 coeff. dell'equazione della circ."""
        x_c, y_c = center.x, center.y
        x_r, y_r = radius.x, radius.y
        self.center = center
        self.radius2 = x_r * x_r + y_r * y_r
        self.a = -2 * x_c
        self.b = -2 * y_c
        self.c = x_c * x_c + y_c * y_c - self.radius2

    def __str__(self):
        """Circle equation."""
        a = px.strcoeff(self.a)
        b = px.strcoeff(self.b)
        c = px.strcoeff(self.c)
        outstr = "x^2 + y^2"
        outstr += " {0}x" if a!='' else '{0}'
        outstr += " {1}y" if b!='' else '{1}'
        outstr += " {2} = 0"
        return outstr.format(a, b, c)

####
# CircleB3P
###

class CircleB3P(Circle):
    """Class for Circle by three Points."""
  
    def __init__(self, point0, point1, point2):
        """Calcola i 3 coeff. dell'equazione della circ, dati 3 punti."""
        xp0, yp0 = point0.x, point0.y
        xp1, yp1 = point1.x, point1.y
        xp2, yp2 = point2.x, point2.y
        self.a, self.b, self.c = eb.solsist((xp0, yp0, 1, -xp0*xp0-yp0*yp0),
                                            (xp1, yp1, 1, -xp1*xp1-yp1*yp1),
                                            (xp2, yp2, 1, -xp2*xp2-yp2*yp2))
        x_c, y_c = (-self.a / 2, -self.b / 2)
        self.center = Point(x_c, y_c)
        self.radius2 = x_c * x_c + y_c * y_c - self.c

####
# Esercizi
###

class _Circle01(eb.Exebase):
    """Base class for Circle 1 exercises."""
    def __init__(self):
        self.delivery = "Equazione della circonferenza e intersezioni"
        self.e_strings = [r'Centro={0};~raggio = sqrt {1}; ~retta:~{2}',
                          r'$$Centro={0}; raggio = sqrt {1}; retta: {2}$$']
        self.r_strings = [(r'{0};~A = {1};~B = {2}'),
                          (r'$${0}; A = {1}; B = {2}$$')]

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.circle.center,
                                                      self.circle.radius2,
                                                      self.line)

    def res2str(self):
        """Response to string."""
        return self.r_strings[self.out_format].format(self.circle,
                                                      self.point0,
                                                      self.point1)

class _Circle02(eb.Exebase):
    """Base class for Circle 2 exercises."""
    def __init__(self):
        self.delivery = "Equazione della circonferenza centro e raggio"
        self.e_strings = [(r'~A = {0};~B = {1};~C = {2}'),
                          (r'$$A = {0}; B = {1}; C = {2}$$')]
        self.r_strings = [r'{0};~Centro={0};~raggio = sqrt {1}',
                          r'$${0}; Centro={0}; raggio = sqrt {1}$$']

    def exe2str(self):
        """Exercises to string."""
        return self.e_strings[self.out_format].format(self.point0,
                                                      self.point1,
                                                      self.point2)

    def res2str(self):
        """Response to string."""
        return self.r_strings[self.out_format].format(self.circle,
                                                      self.circle.center,
                                                      self.circle.radius2)

class Exe01circlelineintersection(_Circle01):
    """Intersezioni circonf.-retta"""
    def __init__(self, val_mi, val_ma):
        _Circle01.__init__(self)
        center = randpoint(val_mi / 2, val_ma / 2)
        radius = randpoint(val_mi / 2, val_ma / 2, [0], [0])
        circle = Circle(center, radius)
        p = center + radius
        x_p, y_p = p.x, p.y
        if x_p == 0:
            q = y_p
            m = eb.rrnoval(val_mi // 4, val_ma // 4, [0])
        else:
            q = eb.rrnoval(val_mi, val_ma)
            m = Fraction(y_p-q, x_p)
        self.line = Linemq(m, q)
        x_a, x_b = eb.eq2solve(float(1 + m * m),
                               float(2 * m * q + circle.a + circle.b * m),
                               float(q * q + circle.b * q + circle.c))
        self.point0 = Point(x_a, self.line.eval(x_a))
        self.point1 = Point(x_b, self.line.eval(x_b))
        self.circle = circle

class Exe02circlelineintersectionrz(_Circle01):
    """Circ. intersezioni con retta (Z)"""
    def __init__(self, val_mi, val_ma):
        _Circle01.__init__(self)
        point0, point1, point2 = three_points_not_in_line(val_mi, val_ma)
        self.circle = CircleB3P(point0, point1, point2)
#        self.center = self.circle.center
        self.line = Line(point0, point1)
        self.point0 = point0
        self.point1 = point1

#class Exe02circlelineintersectionrz(_Circle01):
#    """Circ. intersezioni con retta (Z)"""
#    def __init__(self, val_mi, val_ma):
#        _Circle01.__init__(self)
#        point0, point1, point2 = threePointsNotInLine(val_mi, val_ma)
#        circle = CircleB3P(point0, point1, point2)
#        center = circle.center
#        line0 = Line(point0, point1)
#        line1 = Line(point1, point2)
#        line2 = Line(point2, point0)
#        out_format = eb.Esbase().out_format
#        if out_format == eb.ODT:
#            e_str = ("Centro={0};~raggio = sqrt {1}; ~rette:~{2}~{3}~{4}")
#            r_str = "{0}; ~A = {1}; ~B = {2}; ~C = {3}"
#        elif out_format == eb.TEX:
#            e_str = r'$$'
#            r_str = r'$$'
#        return (e_str.format(center, circle.radius2, line0, line1, line2),
#                r_str.format(circle, point0, point1, point2))

class Exe03circleby3points(_Circle02):
    """Circ. per 3 punti"""
    def __init__(self, val_mi, val_ma):
        _Circle02.__init__(self)
        (self.point0,
         self.point1,
         self.point2) = three_points_not_in_line(val_mi, val_ma)
        self.circle = CircleB3P(self.point0, self.point1, self.point2)

def main():
    """Main program."""
    exercises = (Exe01circlelineintersection,
                 Exe02circlelineintersectionrz,
                 Exe03circleby3points)
    for exe in exercises:
        print('\n', exe.__doc__)
        items = [exe(-10, +10) for _ in range(3)]
        for out_format in range(2):
            print('\nout_format:', out_format)
            items[0].out_format = out_format
            for item in items:
                print(item)

if __name__ == "__main__" :
    main ()
