#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-
#------------------------------python-------------------------genes.py--#
#                                                                       #
#                      Generatore di esercizi                           #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli-------------GPL-------------------------------2014--#

"""
Math exercises generator.
"""

from __future__ import print_function, division
import os, sys
try:
    import Tkinter as tk
    #import FileDialog as fd
    import tkFileDialog as tkfd
    import tkMessageBox as tkmb
except ImportError:
    import tkinter as tk
    import tkinter.filedialog as tkfd
    import tkinter.messagebox as tkmb

#from fract import Fraction
#from polynx import Polynx
#from symbbase import SymBase
from esbase import Workbook

#RIGHEVUOTE=3
LTESTO = 100
HTESTO = 40
COLGRA = '#CFB53B'
COLFRA = '#E6E8FA'
COLGEN = '#8FBC8F'
MI = -12
MA = 12
DIRESE = '/exercises'
FORMATS = ('open document', 'LaTeX') #, 'python', 'text')
LAYOUTS = ('Exe over res', 'Inline', 'Moodle-XML')
MAXCLA = 256

class GeneseForm: 
    """Main class for generator."""

    def __init__(self, master):
        self.master = master
        self.nomeese = ''
        self.selected = tk.IntVar()
        self.format = tk.IntVar()
        self.layout = tk.IntVar()
        self.numese = tk.IntVar(master, 10)
        self.min = tk.IntVar(master, -12)
        self.max = tk.IntVar(master, 12)
        self.num = tk.IntVar(master, 1)
        self.exercises = {}
        self.make_widgets(master)
        self.workbook = Workbook()

    def make_widgets(self, master):
        """Create all widgets."""

        def insor(master, row, col, text, var=None):
            """Inserisce all'interno del master m nella riga r un entry
            formato da un'etichetta t, seguita da un oggetto Entry collegato
            alla variabile tv."""
            sticky = tk.W + tk.E
            width = 5 
            tk.Label(master, text=text, anchor=tk.W).grid(row=row, 
                                                          column=col,
                                                          sticky=sticky)
            entry = tk.Entry(master, width=width, textvariable=var)
            entry.grid(row=row, column=col+1, sticky=sticky)
            return entry

        def make_menufile(mbar):
            """Make menu File."""
            menufile = tk.Menu(mbar)
            menufile.add_command(label='Genera',
                                 underline=0, command=self.generates)
            menufile.add_command(label='Salva',
                                 underline=0, command=self.salva)
            menufile.add_command(label='Sava con nome',
                                   underline=9, command=self.salvacn)
            menufile.add_command(label='Carica',
                                   underline=0, command=self.carica)
            menufile.add_command(label='Cancella',
                                   underline=1, command=self.cancella)
            menufile.add_separator()
            menufile.add_command(label="Exit",
                                 underline=0, command=self.exit)
            return menufile

        def make_menuexe(mbar):
            """Load all math esercises and make menu Ese."""
            menuexe = tk.Menu(mbar)
            nfiles = [n_f[:-3] for n_f in os.listdir(os.curdir+DIRESE) if 
                                                           n_f[-3:] == '.py']
            sys.path.append(sys.path[0] + DIRESE)
            nfiles.sort()
            for i_f, n_f in enumerate(nfiles):
                mod = __import__(n_f)
                submenu = tk.Menu(menuexe)
                classes = [getattr(mod, name) for name in dir(mod)
                                                   if name.startswith('Exe')]
##                classes.sort(lambda x, y: cmp(x.__name__, y.__name__))
                classes.sort(key = lambda x: x.__name__)
                for i_c, cla in enumerate(classes):
                    num_cla = i_f * MAXCLA + i_c
                    submenu.add_radiobutton(label=cla.__doc__,
                                            variable=self.selected,
                                            value=num_cla)
                    self.exercises[num_cla] = cla
                menuexe.add_cascade(label=mod.__doc__.strip(), menu=submenu)
            return menuexe

        def make_menuformat(mbar, formats):
            """Make menu Format."""
            menu = tk.Menu(mbar)
            for i, text in enumerate(formats):
                menu.add_radiobutton(label='{0:2d}. {1}'.format(i, text),
                                           variable=self.format, value=i)
            return menu

        def make_menulayout(mbar, layouts):
            """Make menu Layout."""
            menu = tk.Menu(mbar)
            for i, text in enumerate(layouts):
                menu.add_radiobutton(label='{0:2d}. {1}'.format(i, text),
                                           variable=self.layout, value=i)
            return menu

        def make_text(master, row, col, wtext, htext, color):
            """Create text box."""
            xsb = 0
            ysb = 1
            textb = tk.Text(master, width=wtext, height=htext,
                            background=color, wrap=tk.WORD)
            if xsb:
                xscroll = tk.Scrollbar(master, orient=tk.HORIZONTAL)
                xscroll.grid(row=row+1, column=col, columnspan=2, 
                             sticky=tk.E+tk.W)
                textb['xscrollcommand'] = xscroll.set
                xscroll['command'] = textb.xview
            if ysb:
                yscroll = tk.Scrollbar(master, orient=tk.VERTICAL)
                yscroll.grid(row=row, column=col+1, sticky=tk.N+tk.S)
                textb['yscrollcommand'] = yscroll.set
                yscroll['command'] = textb.yview
            textb.grid(row=row, column=col, sticky=tk.N+tk.S+tk.E+tk.W)
            return textb

        def make_buttons(master, bottons, row=0, col=0, irow=1, icol=0):
            """Create bottons."""
            for bott in bottons:
                button = tk.Button(master, text=bott[0], command=bott[1])
                button.grid(row=row, column=col)
                row += irow
                col += icol

# main createWidgets
        self.master.title("Exercises generator")
# make menu
        menubar = tk.Menu(master)
        master.config(menu=menubar)
        menubar.add_cascade(label="File", underline=0,
                            menu=make_menufile(menubar))      
        menubar.add_cascade(label="Esercizi", underline=0,
                            menu=make_menuexe(menubar))      
        menubar.add_cascade(label="Formato", underline=1,
                            menu=make_menuformat(menubar, FORMATS))      
        menubar.add_cascade(label="Layout", underline=1,
                            menu=make_menulayout(menubar, LAYOUTS))      
# frame di bottoni: comandi
        bottonifr = tk.Frame(master, bd=1, relief=tk.RAISED)
#        bottonifr = tk.Frame(master, background=COLGEN,)
        bottonifr.grid(row=3, column=0, columnspan=3,
                       sticky=tk.N+tk.S+tk.E+tk.W)
        valorifr = tk.Frame(bottonifr,
                            background=COLGRA, relief=tk.RIDGE, bd=3)
        valorifr.grid(ipady=10)
        insor(bottonifr, 0, 0, "numero esercizi:", var=self.numese)
        insor(bottonifr, 0, 2, "minimo:", var=self.min)
        insor(bottonifr, 0, 4, "massimo:", var=self.max)
        insor(bottonifr, 0, 6, "numera da:", var=self.num)
        bottonigra = (('Genera', self.generates),
                    ('Basta!', self.exit),
                    ('Informazioni', self.info),
                   )
        make_buttons(bottonifr, bottonigra, col=8, irow=0, icol=1)
    # textbox: esercizi
        self.esercizitb = make_text(master, 2, 1, LTESTO, HTESTO, COLGRA)

####
# Metodi chiamati in risposta alla pressine dei bottoni
###
    def generates(self):
        """Generates an exercise."""
        self.workbook.out_format = self.format.get()
        exe = self.exercises[self.selected.get()]
        self.workbook.generates(self.numese.get(), exe,
                                self.min.get(), self.max.get())
        self.workbook.num = self.num.get()
        self.workbook.layout = self.layout.get()
        self.esercizitb.insert(tk.END, str(self.workbook) + "\n\n")
        self.num.set(self.workbook.num)

    def salva(self):
        """Save text."""
        if self.nomeese == '':
            ofile = tkfd.asksaveasfile(defaultextension='.ese',
                                      filetypes=[('esercizi', '*.ese')],
                                      title='Salva esercizi')
        else:
            ofile = open(self.nomeese, 'w')
        if ofile is not None:
            gra = self.esercizitb.get('0.1', tk.END).encode('latin-1')
            ofile.write(gra)
            self.nomeese = ofile.name
            self.master.title(self.nomeese)
            ofile.close()

    def salvacn(self):
        """Save as."""
        ofile = tkfd.asksaveasfile(defaultextension='.ese',
                                  filetypes=[('esercizi', '*.ese')],
                                  title='Salva esercizi')
        if ofile is not None:
            gra = self.esercizitb.get('0.1', tk.END).encode('latin-1')
            ofile.write(gra)
            self.nomeese = ofile.name
            self.master.title(self.nomeese)
            ofile.close()

    def carica(self):
        """Load."""
        # TODO
        # se ci sono gi esercizi deve chiedere se la si vuole salvare
        ifile = tkfd.askopenfile(defaultextension='.ese',
                                filetypes=[('esercizi', '*.ese')],
                                title='Apri esercizi')
        if ifile is not None:
        # se c' gi una grammatica deve cancellarla
            self.esercizitb.delete("1.0", tk.END)
            self.esercizitb.insert('1.0', ifile.read())
            self.nomeese = ifile.name
            self.master.title(self.nomeese)
            ifile.close()

    def cancella(self):
        """Clear text."""
        self.esercizitb.delete("1.0", tk.END)
        self.num.set(1)
        self.master.title(self.nomeese)

#    def stampagra(self):
#        """Print text."""
#        print "stampagra"

    def exit(self):
        """Exit."""
        self.master.quit()
        self.master.destroy()

    def info(self):
        """Info."""
        tkmb.showinfo("Informazioni", """
Programma scritto in Python,
rilasciato sotto licenza GPL (vedi: www.gnu.org),
da Daniele Zambelli (daniele.zambelli@inwind.it).
Viene diffuso senza alcuna garanzia
nella speranza che possa servire e divertire.

Sono graditi commenti, critiche, suggerimenti...""")

ROOT = tk.Tk()
GeneseForm(ROOT) 
ROOT.mainloop() 

