#!/usr/bin/python
#-------------------------------python-----------------------esbase.py--#
#                                                                       #
#                    Classe base pe gli esercizi                        #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2013---#


"""
Base class for exercise.
"""

from __future__ import print_function, division
import random
#import math
from fract import Fraction, mcd #, FRA_TYPE
from polynx import Polynx
from symbbase import MultiOutputBase

FRA_TYPE = type(Fraction())
ODT, TEX, PY, TXT = range(4)

####
# Tools
###

def near(num0, num1):
    """num0 near num1."""
    return abs(num0 - num1) < 1e-8

def rrnoval(val_mi, val_ma, excluded=[]):
    """Return a random value not in v."""
    while True:
        result = random.randrange(val_mi, val_ma)
        if not result in excluded:
            return result

def randsign():
    """Return a random sign."""
#    return rrnoval(-1, 2, [0])
    if random.randrange(0, 2):
        return -1
    else:
        return +1
    
def coppiaprimi(val_mi, val_ma, noa=[], nob=[]):
    """Pair of random prime integer."""
    num0 = rrnoval(val_mi, val_ma, noa)
    num1 = rrnoval(val_mi, val_ma, nob)
    div = mcd(num0, num1)
#  print a, b, d
    return num0 / div, num1 / div

def binomio(grado, coe1, coe0):
    """Binomial."""
    return Polynx([coe1] + [0] * (grado - 1) + [coe0])

def odd(num):
    """num is odd."""
    return num % 2

def even(num):
    """num is even."""
    return num % 2 - 1

def divisors(num):
    """Result a tuple with divisors of num."""
    if type(num) == FRA_TYPE:
        num = num.getnum()
    num = abs(num)
    result = []
    div = 1
    while True:
        quo, rem = divmod(num, div)
    #    print d, q, r
        if div > quo:
            break
        if div == quo:
            result.append(div)
        elif rem == 0:
            result.extend([div, quo])
        div += 1
    return result

#def delta(a, b, c):
#    """Discr."""
#    return b * b - 4 * a * c

##def rd(a, b, c):
##  return sqrt(-delta(a, b, c))

def eq2solve(c_a, c_b, c_c):
    """Return solutions of eq2."""
#  print a, b, c
#    d = delta(a, b, c)
    discr = c_b * c_b - 4 * c_a * c_c
#  print d
    if discr >= 0:
        sqrt_discr = discr ** .5
#    print rd
#    print (-b-rd)/(2*a), (-b+rd)/(2*a)
        return ((-c_b - sqrt_discr) / (2 * c_a),
                (-c_b + sqrt_discr) / (2 * c_a))
    else:
        sqrt_discr = (-discr) ** .5
        p_real = Fraction(-c_b, 2 * c_a)
        p_imm = Fraction(sqrt_discr, 2 * c_a)
        if p_imm >= 0:
            return ("{0} + {1}j".format(p_real, p_imm),
                    "{0} - {1}j".format(p_real, p_imm))
        else:
            return ("{0} - {1}j".format(p_real, -p_imm),
                    "{0} + {1}j".format(p_real, -p_imm))
    
def det3(a00, a01, a02, a10, a11, a12, a20, a21, a22):
    """Return det to matrix3.
a00, a01, a02,
a10, a11, a12,
a20, a21, a22
    """
    return (a00 * a11 * a22 + a01 * a12 * a20 + a02 * a10 * a21 - 
            a00 * a12 * a21 - a01 * a10 * a22 - a02 * a11 * a20)

def solsist(eq1, eq2, eq3):
    """Return solutions of linear sist."""
    cx1, cy1, cz1, tn1 = eq1
    cx2, cy2, cz2, tn2 = eq2
    cx3, cy3, cz3, tn3 = eq3
    det0 = det3(cx1, cy1, cz1, cx2, cy2, cz2, cx3, cy3, cz3)
    if det0:
        detx = det3(tn1, cy1, cz1, tn2, cy2, cz2, tn3, cy3, cz3)
        dety = det3(cx1, tn1, cz1, cx2, tn2, cz2, cx3, tn3, cz3)
        detz = det3(cx1, cy1, tn1, cx2, cy2, tn2, cx3, cy3, tn3)
        return (Fraction(detx, det0),
                Fraction(dety, det0),
                Fraction(detz, det0))
    else:
        return (None, None, None)

####
# Base class ofr Execises
###

class EsbaseError(Exception):
    """Fraction error."""



class Esbase(MultiOutputBase):
    """Base class for execrises."""

    layout = 1

    def __init__(self):
        self.num = 0
        self.ese = []
        self.res = []

    def __str__(self):
        """str."""
        return [self._str_odt,
                self._str_tex,
                self._str_tex,
                self._str_tex][self.out_format]()

    def _str_odt(self):
        """.odt string."""
        if Esbase.layout == 0:
            lines = []
            for ese, res in zip(self.ese, self.res):
                lines.append('"{0:3}.  " {1} = {2}'.format(self.num, 
                                                           ese, res))
                self.num += 1
        elif Esbase.layout == 1:
            lines = []
            num = self.num
            for ese in self.ese:
                lines.append('"{0:3}.  " {1}'.format(num, ese))
                num += 1
            lines.append('newline "soluzioni:"')
            for res in self.res:
                lines.append('"{0:3}.  " {1}'.format(self.num, res))
                self.num += 1
        else:
            raise EsbaseError('{0} layout not supported'.format(Esbase.layout))
        return " newline\n".join(lines)
        

    def _str_tex(self):
        """.tex string."""
        if Esbase.layout == 0:
            lines = []
            for ese, res in zip(self.ese, self.res):
                lines.append(r'{0:3}. {1} = {2}'.format(self.num, ese[:-1],
                                                                  res[1:]))
                self.num += 1
        elif Esbase.layout == 1:
            lines = []
            num = self.num
            for ese in self.ese:
                lines.append(r'{0:3}. {1}'.format(num, ese))
                num += 1
            lines.append('\n\nsoluzioni:')
            for res in self.res:
                lines.append(r'{0:3}. {1}'.format(self.num, res))
                self.num += 1
        else:
            raise EsbaseError('{0} layout not supported'.format(Esbase.layout))
        return "\n\n".join(lines)


#    def _str_tex(self):
#        """.tex string."""
#        if Esbase.layout == 0:
#            lines = ['{0} = {1}'.format(e, r) for e, r in zip(self.ese, 
#                                                              self.res)]
#        elif Esbase.layout == 1:
#            lines = self.ese + ['\n\n "soluzioni:"'] + self.res
#        else:
#            raise EsbaseError('{0} layout not supported'.format(Esbase.layout))
#        return "\n\n".join(lines)

#    def genera(self):
#        """To redefine."""
#        return None, None
#
#    def es_sol(self, genera_function):
#        """Return a exercise follwed by solution."""
#        self.genera()
#        return '{0} = {1}'.format(self.ese, self.res)

    def genera_n(self, genera_function, val_mi, val_ma, num):
        """Return n exercises follwed by n solutions."""
        self.ese = []
        self.res = []
        for _ in range(num):
            ese, res = genera_function(val_mi, val_ma)
            self.ese.append(ese)
            self.res.append(res)

#    def es_sol_n(self, num, n_from = 1):
#        """Return n exercises follwed by n solutions."""
#        eselist = []
#        reslist = []
#        for i in range(n_from, n_from + num):
#            ese, res = self.genera()
#            eselist.append('"{0:3}.  " {1}'.format(i, ese))
#            reslist.append('"{0:3}.  " {1}'.format(i, res))
#        result = " newline\n".join(eselist +
#                                   ['newline "soluzioni:"'] + reslist)
#        return result

def main():
    """Main."""
    for _ in range(20):
        print(coppiaprimi(-12, 12, (0,), (0,)))

if __name__ == "__main__" :
    main ()
