#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
#-------------------------------python--------------------------eq2.py--#
#                                                                       #
#                     Equazioni di secondo grado                        #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, val_ma  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2006---#

"""
Eq. di secondo grado
"""

if __name__ == "__main__" :
  import sys
  sys.path.append(sys.path[0][:-9])

import esbase as eb
#import random
from fract import Fraction
from polynx import Polynx

def result_eq2(exe, res0, res1):
    out_format = eb.Esbase().out_format
    if out_format == eb.ODT:
        e_str = '{0} = 0'
        r_str = 'left [{0}; {1} right ]'
    elif out_format == eb.TEX:
        e_str = r'${0} = 0$'
        r_str = r'$\left [{0}; {1} \right ]$'
    return (e_str.format(exe),
            r_str.format(res0, res1))

#def exe_02_eq2cq(exe, res0, res1):  # TODO implementare per le equazioni con coefficienti razinali
#    co = random.choice(e.coeff)
#    if co:
#      di = random.choice(eb.divisors(co))
#      e/=di
#    Eq2.genera()

def exe_01_eq2rzcz(val_mi, val_ma):
    """Eq. 2gr: rad. intere, coeff. interi"""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    return result_eq2(Polynx([1, a]) * Polynx([1, b]), -a, -b)

def exe_02_eq2rqcz(val_mi, val_ma):
    """Eq. 2gr: rad. razionali, coeff. interi"""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    c = eb.rrnoval(0, val_ma, [0])
    d = eb.rrnoval(val_mi, val_ma, [0])
    return result_eq2(Polynx([a, b]) * Polynx([c, d]),
                      Fraction(-b, a), Fraction(-d, c))

def exe_03_eq2rzcq(val_mi, val_ma):
    """Eq. 2gr: rad. intere, coeff. razionali"""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    fac = eb.rrnoval(1, val_ma, [0])
    div = eb.rrnoval(1, val_ma, [0])
    pol0 = Polynx([Fraction(1 * fac, div), Fraction(a * fac, div)])
    pol1 = Polynx([1, b])
    return result_eq2(pol0 * pol1, -a, -b)

def exe_04_eq2rqcq(val_mi, val_ma):
    """Eq. 2gr: rad. razionali, coeff. razionali"""
    a = eb.rrnoval(1, val_ma)
    b = eb.rrnoval(val_mi, val_ma, [0])
    c = eb.rrnoval(1, val_ma)
    d = eb.rrnoval(val_mi, val_ma, [0])
    pol0 = Polynx([Fraction(a, b), 1])
    pol1 = Polynx([1, Fraction(d, c)])
    return result_eq2(pol0 * pol1, -a, -b)

def exe_05_eq2rccz(val_mi, val_ma):
    """Eq. 2gr: rad. complesse, coeff. interi"""
    a = eb.rrnoval(1, val_ma)
    bre = eb.rrnoval(val_mi, val_ma)
    bim = eb.rrnoval(val_mi, val_ma, [0])
    sumx = bre * 2 * a
    prox = bre * bre + bim * bim
    xre = Fraction(bre, a)
    xim = Fraction(bim, a)
    return result_eq2(Polynx([a * a, -sumx, prox]),
                      '{0} - {1} i'.format(xre, xim),
                      '{0} + {1} i'.format(xre, xim))

def exe_06_eq2rccq(val_mi, val_ma):
    """Eq. 2gr: rad. complesse, coeff. razionali"""
    a = eb.rrnoval(1, val_ma)
    bre = eb.rrnoval(val_mi, val_ma)
    bim = eb.rrnoval(val_mi, val_ma, [0])
    sumx = bre * 2 * a
    prox = bre * bre + bim * bim
    xre = Fraction(bre, a)
    xim = Fraction(bim, a)
    return result_eq2(Polynx([a * a, -sumx, prox]),
                      '{0} - {1} i'.format(xre, xim),
                      '{0} + {1} i'.format(xre, xim))

def main():
    exe=(exe_01_eq2rzcz, exe_02_eq2rqcz,
         exe_03_eq2rzcq, exe_04_eq2rqcq,
         exe_05_eq2rccz, exe_06_eq2rccq)
    for ese in exe:
        print(ese.__doc__)
        for out_format in range(2):
            print('out_format:', out_format)
            exbas = eb.Esbase()
            exbas.out_format = out_format
            exbas.genera_n(ese, -10, +10, 5)
            for layout in range(2):
                print('layout:', layout)
                eb.Esbase.layout = layout
                print(exbas)
                print()
    

if __name__ == "__main__" :
    main ()
