#!/usr/bin/python
#-------------------------------python----------------------prodnot.py--#
#                                                                       #
#                         Prodotti notevoli                             #
#                                                                       #
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2013---#

"""
Prodotti notevoli
"""

from __future__ import print_function, division

if __name__ == "__main__" :
    import sys
    sys.path.append(sys.path[0][:-9])

import esbase as eb
import random
import polynx as px

def exe_01_somdif(val_mi, val_ma):
    """Somma per differenza: (ax+b)(ax-b)."""
    a = eb.rrnoval(val_mi, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    if random.randrange(2) == 0:
        c, d = a, -b
    else:
        c, d = -a, b
    pol0 = px.Polynx([a, b])
    pol1 = px.Polynx([c, d])
    if eb.Esbase.out_format == eb.ODT:
        e_str = 'left ( {0} right ) left ({1} right )'
        r_str = '{0}'
    elif eb.Esbase.out_format == eb.TEX:
        e_str = r'$\left({0} \right) \left({1} \right)$'
        r_str = '${0}$'
    return (e_str.format(pol0, pol1),
            r_str.format(pol0 * pol1))

def exe_02_quadbin(val_mi, val_ma):
    """Quadrato di un binomio: (ax+b)^2."""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    pol0 = px.Polynx([a, b])
    prod = pol0 * pol0
    if eb.Esbase.out_format == eb.ODT:
        ese = 'left ({0} right )^2'.format(pol0)
        res = '{0}'.format(prod)
    elif eb.Esbase.out_format == eb.TEX:
        ese = r'$\left({0} \right)^2$'.format(pol0)
        res = '${0}$'.format(prod)
    return ese, res

def exe_03_quadtrin(val_mi, val_ma):
    """Quadrato di un trinomio: (ax^2+bx+c)^2."""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    c = eb.rrnoval(val_mi, val_ma, [0])
    pol0 = px.Polynx([a, b, c])
    prod = pol0 * pol0
    if eb.Esbase.out_format == eb.ODT:
        ese = 'left ({0} right )^2'.format(pol0)
        res = '{0}'.format(prod)
    elif eb.Esbase.out_format == eb.TEX:
        ese = r'$\left({0} \right)^2$'.format(pol0)
        res = '${0}$'.format(prod)
    return ese, res

def exe_04_trinnot(val_mi, val_ma):
    """Trinomio particolare: (x+a)(x+b)."""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    pol0 = px.Polynx([1, a])
    pol1 = px.Polynx([1, b])
    prod = pol0 * pol1
    if eb.Esbase.out_format == eb.ODT:
        ese = 'left ( {0} right ) left ({1} right )'.format(pol0, pol1)
        res = '{0}'.format(prod)
    elif eb.Esbase.out_format == eb.TEX:
        ese = r'$\left({0} \right) \left({1} \right)$'.format(pol0, pol1)
        res = '${0}$'.format(prod)
    return ese, res

def exe_05_cubbin(val_mi, val_ma):
    """Cubo di un binomio: (ax+b)^3."""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    pol0 = px.Polynx([a, b])
    prod = pol0 * pol0 * pol0
    if eb.Esbase.out_format == eb.ODT:
        ese = 'left ({0} right )^3'.format(pol0)
        res = '{0}'.format(prod)
    elif eb.Esbase.out_format == eb.TEX:
        ese = r'$\left({0} \right)^3$'.format(pol0)
        res = '${0}$'.format(prod)
    return ese, res

def exe_06_potbin(val_mi, val_ma):
    """Potenza di un binomio: (ax+b)^n con n in 3-5."""
    a = eb.rrnoval(0, val_ma, [0])
    b = eb.rrnoval(val_mi, val_ma, [0])
    pol0 = px.Polynx([a, b])
    prod = pol0
    esp = random.randrange(3, 6)
    for _ in range(esp-1):
        prod *= pol0
    if eb.Esbase.out_format == eb.ODT:
        ese = 'left ({0} right )^{1}'.format(pol0, esp)
        res = '{0}'.format(prod)
    elif eb.Esbase.out_format == eb.TEX:
        ese = r'$\left({0} \right)^{1}$'.format(pol0, esp)
        res = '${0}$'.format(prod)
    return ese, res

def main():
    """Main program."""
    exe = (exe_01_somdif,  exe_02_quadbin, 
           exe_03_quadtrin, exe_04_trinnot, 
           exe_05_cubbin, exe_06_potbin)
    for ese in exe:
        print(ese.__doc__)
        for out_format in range(2):
            print('out_format:', out_format)
            eb.Esbase.out_format = out_format
            exbas = eb.Esbase()
            exbas.genera_n(ese, -10, +10, 5)
            for layout in range(2):
                print('layout:', layout)
                eb.Esbase.layout = layout
                print(exbas)
                print()
    
if __name__ == "__main__" :
    main ()
