#!/usr/bin/python
#-------------------------------python-----------------------polinx.py--#
#                                                                       #
#                     Polinomi in una variabile                         #
#                                                                       #
# Copyright (c) 2013 Daniele Zambelli
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#--Daniele Zambelli--------------GPL-----------------------------2014---#

"""
One variable Polynomials
"""
from __future__ import print_function, division
from fract import Fraction #, FORMATS
from symbbase import MultiOutputBase
import random

VERSION = '0.8'

####
# Tools
###

##def segnoass(n):
##  if n<0: return '-', -n
##  else  : return '+', n
##
##def segnoass_no_zero(n):
##  if n<0  : return '-', -n
##  elif n>0: return '+', n
##  else    : return '', n


def strcoeff(coef, prime=False):
    """Restituisce il coefficiente adatto."""
#    print(coef, prime)
    if coef == 0:
        return ''
    elif coef == -1:
        return '-'
    elif coef == 1:
        return '' if prime else '+'
    elif coef > 0:
        return ('' if prime else '+') + str(coef)
    else:
        return str(coef)

##def coeff_no_uno(n):
##  s, a = segnoass(n)
##  if a==1: return s
##  else   : return s + ' ' + str(a)


def toglipiu(string):
    """Toglie l'eventuale + davanti al coefficiente."""
    if string.startswith('+ '):
        return string[2:]
    elif string.startswith('+'):
        return string[1:]
    else:
        return string


def reducecoeff(coeff_list):
    """Toglie i coefficienti di grado maggiore se uguali a zero."""
    try:
        i = 0
        while i < len(coeff_list) - 1 and coeff_list[i] == 0:
            i += 1
        return coeff_list[i:]
    except TypeError:
        coeff_list += 0
        return [coeff_list]

####
# Classes
###


class NotYetImplementedError(Exception):
    """Not Yet Implemented Exception."""
    pass


class Polynx(MultiOutputBase):
    """Classe che implementa polinomi in una variabile a coefficienti interi"""

    var = 'x'   # Variabile del Polynx

    def __init__(self, coeff=None):
        """Crea un polinomio.
>>> p = Polynx()
>>> print(p.coeff)
[0]
>>> p = Polynx(7)
>>> print(p.coeff)
[7]
>>> p = Polynx([0])
>>> print(p.coeff)
[0]
>>> p = Polynx([0, 0, 0, 0])
>>> print(p.coeff)
[0]
>>> print(Polynx([7]).coeff)
[7]
>>> print(Polynx([-7]).coeff)
[-7]
>>> print(Polynx([5, 4, 3, 2, 1]).coeff)
[5, 4, 3, 2, 1]
>>> print(Polynx([-8, 0, 0, 0, 1]).coeff)
[-8, 0, 0, 0, 1]
>>> print(Polynx([0, 0, 0, 0, -8, 0, 0, 0, 1]).coeff)
[-8, 0, 0, 0, 1]
        """
        if coeff is None:
            self.coeff = [0]
        else:
#            self.coeff.reverse()
            self.coeff = reducecoeff(coeff)

    def __str__(self):
        """str.
>>> p0 = Polynx()
>>> p1 = Polynx([0])
>>> p2 = Polynx([0, 0, 0, 0])
>>> print(p0, p1, p2)
0 0 0
>>> p0 == p1 and p1 == p2
True
>>> print(p0)
0
>>> print(Polynx([7]))
7
>>> print(Polynx([-7]))
-7
>>> Polynx.out_format = ODT
>>> print(Polynx([1, 4, 3, 2, 7]))
x^4 +4 x^3 +3 x^2 +2 x +7
>>> print(Polynx([-1, 4, 3, 2, 7]))
- x^4 +4 x^3 +3 x^2 +2 x +7
>>> print(Polynx([5, 4, 3, 2, 7]))
5 x^4 +4 x^3 +3 x^2 +2 x +7
>>> Polynx.out_format = TEX
>>> print(Polynx([5, 4, 3, 2, 7]))
5 x^4 +4 x^3 +3 x^2 +2 x +7
>>> Polynx.out_format = PY
>>> print(Polynx([5, 4, 3, 2, 7]))
5*x**4 +4*x**3 +3*x**2 +2*x +7
>>> Polynx.out_format = ODT
>>> print(Polynx([-5, 4, -3, 2, -7]))
-5 x^4 +4 x^3 -3 x^2 +2 x -7
>>> Polynx.out_format = TEX
>>> print(Polynx([-5, 4, -3, 2, -7]))
-5 x^4 +4 x^3 -3 x^2 +2 x -7
>>> Polynx.out_format = PY
>>> print(Polynx([-5, 4, -3, 2, -7]))
-5*x**4 +4*x**3 -3*x**2 +2*x -7
>>> Polynx.out_format = ODT
>>> print(Polynx([-8, 0, 0, 0, 7]))
-8 x^4 +7
>>> Polynx.out_format = TEX
>>> print(Polynx([-8, 0, 0, 0, 7]))
-8 x^4 +7
>>> Polynx.out_format = PY
>>> print(Polynx([-8, 0, 0, 0, 7]))
-8*x**4 +7
>>> Polynx.out_format = TEX
>>> print(Polynx([Fraction(3, 2), Fraction(10, 8)]))
\\frac{3}{2} x +\\frac{5}{4}
>>> Polynx.out_format = PY
>>> print(Polynx([Fraction(3, 2), Fraction(10, 8)]))
3/2*x +5/4
>>> Polynx.out_format = TEX
        """
        Fraction.out_format = Polynx.out_format
        return [self._str_odt,
                self._str_tex,
                self._str_py][self.out_format]()

    def coeff2str(self):
        """Convert coefficient to string."""
        if len(self.coeff) == 1:
            return [str(self.coeff[0])]
        else:
            result = [strcoeff(coe) for coe in self.coeff]
            if len(result[-1]) == 1:
                result[-1] = result[-1] + '1'
            return result

    def _str(self, time_sym, pow_sym):
        """String constructor."""
        var = Polynx.var
        coeff = self.coeff2str()
#        coeff.reverse()
        result = ''
        length = len(coeff) - 1
        for i, coe in enumerate(coeff):
            if coe:
                if len(coe) == 1:
                    perc = ' '
                else:
                    perc = time_sym
                if i < length - 1:
                    potc = pow_sym + str(length - i) + ' '
                elif i == length - 1:
                    potc = ' '
                else:
                    perc = var = potc = ''
                result += coe + perc + var + potc
        return toglipiu(result)

    def _str_odt(self):
        """.odt string."""
        return self._str(' ', '^')

    def _str_tex(self):
        """.txt string."""
        return self._str(' ', '^')

    def _str_py(self):
        """.tex string."""
        return self._str('*', '**')

    def eval(self, xval):
        """Return P(x).
>>> p = Polynx([1, 0, 0, -8])
>>> print(p.eval(2))
0.0
>>> p = Polynx([3, 0, 5, 7])
>>> print(p.eval(1))
15.0
        """
        coeff = list(self.coeff)
#        coeff.reverse()
        result = 0.
        for coe in coeff:
            result = result * xval + coe
        return result

    def __cmp__(self, other):
        """Confronta due polinomi o un Polynx e un intero.
>>> p0 = Polynx([1, 0, 0, -8])
>>> p1 = Polynx([0, 0, 0, 1, 0, 0, -8])
>>> p2 = Polynx([1, 0, 0, 8])
>>> print(p0 == p1)
True
>>> print(p0 == p2)
False
>>> print(p0 == 3)
True
        """
        if type(other) == INT_TYPE:
            return self.coeff == [other]
        else:
            return cmp(self.coeff, other.coeff)

    def __neg__(self):
        """Restituisce l'opposto di un Polynx.coeff.
>>> print(-Polynx([3, -5, 7, -2]))
-3 x^3 +5 x^2 -7 x +2
        """
        coeff = [-e for e in self.coeff]
        return Polynx(coeff)

    def __add__(self, other):
        """Restituisce la somma di due polinomi o di un Polynx e un intero.
>>> print(Polynx([3, -5, 7, -2]) + Polynx([3, 5, 7]))
3 x^3 -2 x^2 +12 x +5
>>> print(Polynx([9, 4, 3, -5, 7, -2]) + Polynx([-9, -4, 0, 3, 5, 7]))
3 x^3 -2 x^2 +12 x +5
>>> print(Polynx([3, -5, 7, -2]) + 5)
3 x^3 -5 x^2 +7 x +3
>>> p0 = Polynx([Fraction(3, 2), Fraction(10, 8)])
>>> p1 = Polynx([Fraction(2, 4), Fraction(3, 7), Fraction(8, 6)])
>>> Polynx.out_format = TEX
>>> print(p0 + p1)
\\frac{1}{2} x^2 +\\frac{27}{14} x +\\frac{31}{12}
        """
        try:
            if len(self.coeff) > len(other.coeff):
                coeff0, coeff1 = other.coeff, self.coeff
            else:
                coeff0, coeff1 = self.coeff, other.coeff
            lenhead = len(coeff1) - len(coeff0)
            head = coeff1[:lenhead]
            coeff1 = coeff1[lenhead:]
            coeff = head + [coe0 + coe1 for coe0, coe1 in zip(coeff0, coeff1)]
        except AttributeError:
#            try:
            coeff = self.coeff[:]
            coeff[-1] = coeff[-1] + other
        return Polynx(coeff)

    def __sub__(self, other):
        """Rest. la differenza di due polinomi o di un Polynx e un intero.
>>> Polynx.out_format = TEX    # perche'??????????????????
>>> print(Polynx([3, -5, 7, -2]) - Polynx([3, 5, 7]))
3 x^3 -8 x^2 +2 x -9
>>> print(Polynx([9, 4, 3, -5, 7, -2]) - Polynx([+9, +4, 0, 3, 5, 7]))
3 x^3 -8 x^2 +2 x -9
>>> print(Polynx([3, -5, 7, -2]) - 5)
3 x^3 -5 x^2 +7 x -7
        """
        return self.__add__(-other)

    def __mul__(self, other):
        """Rest. il prodotto di due polinomi o di un Polynx e un intero.
>>> print(Polynx([3, -5]) * Polynx([3, 5]))
9 x^2 -25
>>> print(Polynx([1, -2]) * Polynx([1, 7]))
x^2 +5 x -14
>>> print(Polynx([3, -5, 7, -2]) * 5)
15 x^3 -25 x^2 +35 x -10
        """
        result = Polynx()
        if type(other) == int:
            result.coeff = [coe * other for coe in self.coeff]
        else:
            coe1, coe2, cpar = self.coeff, other.coeff, []
            res = [coe2[0] * e for e in coe1]
            for coe in coe2[1:]:
                cpar += [0]
                respar = cpar + [coe * e for e in coe1]
                res = ([e1 + e2 for e1, e2 in zip(res, respar[:-1])] +
                       respar[-1:])
                result.coeff = res
        return result

    def __div__(self, other):
        """Restituisce il quoziente di un polinomio per un intero.
        Non ancora implementata la divisone di polinomi"""
        result = Polynx()
        if type(other) == POL_TYPE:
            raise NotYetImplementedError()
        else:
    ##      for c in self.coeff:
    ##        print "%s/%s=%s" % (c, other, c/other)
    ##        print "%s/%s=%s" % (c, other, Fraction(c)/other)
            result.coeff = [Fraction(c) / other for c in self.coeff]
            return result

    def __radd__(self, other):
        """Restituisce la somma di un intero e un polinomo.
>>> print(5 + Polynx([3, -5, 7, -2]))
3 x^3 -5 x^2 +7 x +3
        """
        return self.__add__(other)

    def __rsub__(self, other):
        """Restituisce la differenza di un intero e un polinomo.
>>> print(5 - Polynx([3, -5, 7, -2]))
-3 x^3 +5 x^2 -7 x +7
        """
        return - self.__sub__(other)

    def __rmul__(self, other):
        """Restituisce la differenza di un intero e un polinomo.
>>> print(5 * Polynx([3, -5, 7, -2]))
15 x^3 -25 x^2 +35 x -10
        """
        return self.__mul__(other)

INT_TYPE = type(0)
FRA_TYPE = type(Fraction())
POL_TYPE = type(Polynx())


def randpoli(maxcoefficiente, mingrado=0, maxgrado=5):
    """
#>>> print("\nPolinomi casuali")
#>>> for i in xrange(10):
#...     print_all_format("", poliacaso(20, 2, 7))
    """
    return Polynx(random.sample(xrange(maxcoefficiente),
                                random.randrange(mingrado, maxgrado + 1)) +
                  [random.randrange(maxcoefficiente) + 1])


if __name__ == "__main__":
    ODT, TEX, PY, TXT = range(4)
    import doctest
    doctest.testmod(verbose=False)